package fi.fa.opplugin;

import com.alibaba.druid.util.StringUtils;
import fi.fa.opplugin.utils.ChangeDataUtils;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.args.BeginOperationTransactionArgs;
import kd.bos.exception.KDBizException;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import java.util.List;

//资产报废单中间表下推业务处理单
public class FaDisCardMidOpPlugin extends AbstractOperationServicePlugIn {
    @Override
    public void beginOperationTransaction(BeginOperationTransactionArgs e) {
        super.beginOperationTransaction(e);
        DynamicObject[] dynamicObjects = e.getDataEntities();
        for (int i = 0; i < dynamicObjects.length; i++) {
            if (StringUtils.equals(e.getOperationKey(), "push")) {
                DynamicObject info = BusinessDataServiceHelper.loadSingle(dynamicObjects[i].getPkValue(), dynamicObjects[i].getDynamicObjectType().getName());
                String nckd_flag = info.getString("nckd_flag");
                if ("1".equals(nckd_flag)) {
                    throw new KDBizException("单据" + info.getString("number") + "已经下推生成了资产调拨单，不允许重复下推!");
                }else {
                    String dispatchType = info.getString("nckd_dispatch_type");
                    String error = ChangeDataUtils.check(info);
                    if (!StringUtils.isEmpty(error)) {
                        this.operationResult.setShowMessage(true);
                        this.operationResult.setSuccess(false);
                        this.operationResult.setMessage(error);
                    }else {
                        List<String> codeList = ChangeDataUtils.getCardNumber(info);
                        if ("2".equals(dispatchType)) {
                            DynamicObject busProcess = BusinessDataServiceHelper.newDynamicObject("nckd_ywcld");
                            busProcess.set("nckd_bizdate", info.getDate("nckd_date_out"));//业务日期
                            busProcess.set("nckd_detailtype", "D");//业务类型 D:报废
                            busProcess.set("nckd_costcompany", ChangeDataUtils.getOrg(info.getString("nckd_org_in")));//资产组织
                            busProcess.set("nckd_textfield1", info.getString("nckd_reason"));//摘要事由
                            busProcess.set("nckd_applierv", ChangeDataUtils.getUser(info.getString("nckd_user_out")));//制单人

                            //制单信息
                            busProcess.set("creator", ChangeDataUtils.getUser(info.getString("nckd_user_out")));//创建人
                            busProcess.set("createtime", info.getDate("nckd_date_out"));//创建时间
                            busProcess.set("billstatus", "A");//状态
                            QFilter filter = new QFilter("number", QCP.in,codeList);
                            filter.and("isbak",  QCP.equals, "0");
                            DynamicObject[] card = BusinessDataServiceHelper.load("fa_card_real","id",new QFilter[] {filter});
                            for (int c = 0; c < card.length; c++) {
                                DynamicObject cardInfo = BusinessDataServiceHelper.loadSingle(card[c].getPkValue(), card[c].getDynamicObjectType().getName());
                                //创建报废明细分录
                                DynamicObjectCollection entryEntity = busProcess.getDynamicObjectCollection("nckd_entryentity6");
                                DynamicObject entry1 = entryEntity.addNew();
                                entry1.set("nckd_basedatafield", cardInfo);//资产编码
                                String nckd_count = info.getString("nckd_count").isEmpty() ? "1" : info.getString("nckd_count");
                                Integer count = Integer.valueOf(nckd_count);
                                entry1.set("nckd_integerfield", count);//报废数量
                            }
                            //执行保存-提交-审核
                            OperationResult saveResult = SaveServiceHelper.saveOperate("nckd_ywcld", new DynamicObject[]{busProcess}, OperateOption.create());
                            if (saveResult.isSuccess()){
                                info.set("nckd_flag", "1");
                                SaveServiceHelper.save(new DynamicObject[]{info});
                                this.operationResult.setShowMessage(true);
                                this.operationResult.setSuccess(true);
                                this.operationResult.setMessage("下推业务处理单成功");
                                OperationResult submitResult = OperationServiceHelper.executeOperate("submit", "nckd_ywcld",saveResult.getSuccessPkIds().toArray(),OperateOption.create());
                                OperationResult auditResult = OperationServiceHelper.executeOperate("audit", "nckd_ywcld",submitResult.getSuccessPkIds().toArray(),OperateOption.create());
                            }else {
                                this.operationResult.setShowMessage(true);
                                this.operationResult.setSuccess(false);
                                this.operationResult.setMessage("下推业务处理单失败");
                            }
                        }
                    }
                }
            }
        }
    }
}
